package editor;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileReader;

import javax.swing.JTextPane;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DocumentFilter;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.TabSet;
import javax.swing.text.TabStop;

/**
 * Die TextPane, in der der Benutzer Text eingeben kann.
 * 
 * @author Dietrich Boles, Uni Oldenburg
 * @version 1.0 (12.11.2008)
 * 
 */
public class EditorPanel extends JTextPane implements UndoableEditListener {

	Editor editor;
	JavaDocument doc;
	InputFilter inputFilter;
	int numberOfLines;

	public EditorPanel(Editor editor) {
		this.editor = editor;
		this.setEditorKit(new StyledEditorKit());
		if (editor.isJava) {
			this.doc = new JavaDocument(editor);
		} else {
			this.doc = new TextDocument(editor);
		}
		this.setDocument(this.doc);
		this.setBackground(Color.WHITE);
		this.setTabs(4);
		this.numberOfLines = 0;
	}
	
	public JavaDocument getJavaDocument() {
		return doc;
	}

	public void setSize(Dimension d) {
		if (d.width < this.getParent().getSize().width) {
			d.width = this.getParent().getSize().width;
		}
		// kein LineWrap
		super.setSize(d);
	}

	public boolean getScrollableTracksViewportWidth() {
		return false; // kein LineWrap
	}

	public void loadFile(String file) {
		try {
			File f = new File(file);
			FileReader fr = new FileReader(f);
			char[] buffer = new char[(int) f.length()];
			fr.read(buffer);
			fr.close();
			this.doc.setDocumentFilter(null);
			this.doc.removeDocListener();
			this.setText(new String(buffer));
			this.doc.addDocListener();

			this.inputFilter = new InputFilter(this);
			this.doc.setDocumentFilter(this.inputFilter);
			this.setCaretPosition(0);
			this.calcNumberOfLines();
			this.editor.getUndoManager().discardAllEdits();
			this.editor.setChanged(false);
		} catch (Throwable exc) {
			exc.printStackTrace();
		}
	}

	private void setTabs(int charactersPerTab) {
		FontMetrics fm = this.getFontMetrics(this.getFont());
		int charWidth = fm.charWidth('w');
		int tabWidth = charWidth * charactersPerTab;

		TabStop[] tabs = new TabStop[100];
		tabs[0] = new TabStop(tabWidth - charWidth);

		for (int j = 1; j < tabs.length; j++) {
			tabs[j] = new TabStop(tabs[j - 1].getPosition() + tabWidth);
		}

		TabSet tabSet = new TabSet(tabs);
		SimpleAttributeSet attributes = new SimpleAttributeSet();
		StyleConstants.setTabSet(attributes, tabSet);
		int length = this.doc.getLength();
		this.doc.setParagraphAttributes(0, length, attributes, true);
	}

	public void undoableEditHappened(UndoableEditEvent e) {
		if (!this.doc.isHighlighting()) {
			this.editor.getUndoManager().addEdit(e.getEdit());
		}
	}

	public int getNumberOfLines() {
		return this.numberOfLines;
	}

	public void calcNumberOfLines() {
		int oldNumberOfLines = this.numberOfLines;
		this.numberOfLines = 1;
		try {
			StringBuffer text = new StringBuffer(this.doc.getText(0, this.doc
					.getLength()));
			for (int i = 0; i < text.length(); i++) {
				if (text.charAt(i) == '\n') {
					this.numberOfLines++;
				}
			}
			if (oldNumberOfLines != this.numberOfLines) {
				this.editor.getLineNumberPanel().setNumberOfLines(
						this.numberOfLines);
			}
		} catch (BadLocationException e) {
			e.printStackTrace();
			this.numberOfLines = oldNumberOfLines;
		}
	}
}

class InputFilter extends DocumentFilter {
	private EditorPanel editorPanel;

	InputFilter(EditorPanel p) {
		this.editorPanel = p;
	}

	public void remove(DocumentFilter.FilterBypass fb, int offset, int length)
			throws BadLocationException {
		super.remove(fb, offset, length);
		this.editorPanel.editor.setChanged(true);
		this.editorPanel.calcNumberOfLines();
	}

	public void insertString(DocumentFilter.FilterBypass fb, int offset,
			String string, AttributeSet attr) throws BadLocationException {
		super.insertString(fb, offset, string, attr);
		this.editorPanel.editor.setChanged(true);
		this.editorPanel.calcNumberOfLines();
	}

	public void replace(FilterBypass fb, int offs, int length, String str,
			AttributeSet a) throws BadLocationException {
		if (str != null && str.charAt(str.length() - 1) == KeyEvent.VK_ENTER && this.editorPanel.editor.isIndent()) {
			String blanks = this.genBlanks(this.editorPanel.getDocument()
					.getText(0, this.editorPanel.getCaretPosition()),
					this.editorPanel.getCaretPosition() - 1);
			str = str + blanks;
		}

		super.replace(fb, offs, length, str, a);
		this.editorPanel.editor.setChanged(true);
		this.editorPanel.calcNumberOfLines();
	}

	private String genBlanks(String text, int pos) {
		int retIndex = -1;
		int i = pos;
		for (; i >= 0; i--) {
			if (text.charAt(i) == '\n') {
				retIndex = i;
				break;
			}
		}
		if (i != -1 && retIndex == -1) {
			return "";
		}
		String blanks = "";
		i = retIndex + 1;
		while (i <= pos) {
			if (text.charAt(i) == ' ') {
				blanks += " ";
			} else if (text.charAt(i) == '\t') {
				blanks += "\t";
			} else {
				break;
			}
			i++;
		}
		return blanks;
	}

}
