package simulation;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenuItem;

import model.Play;
import simulation.listener.EventObject;
import theater.Actor;
import theater_intern.IPerformance;
import theater_intern.IStage;
import theater_intern.TheaterObserver;
import util.ResourceManager;

public class RunSimulationManager extends SimulationManager implements
		ActionListener {

	public final static int WAIT_TIME = 5000;

	private JButton startButton;
	private JButton pauseButton;
	private JButton stopButton;
	private JButton resetButton;
	private JMenuItem startItem;
	private JMenuItem pauseItem;
	private JMenuItem stopItem;
	private JMenuItem resetItem;

	private Vector<EventObject> eventObjects;
	private Object eventSync;

	private RunSimulator simulator;

	public RunSimulationManager(JButton startButton, JButton pauseButton,
			JButton stopButton, JButton resetButton, JMenuItem startItem,
			JMenuItem pauseItem, JMenuItem stopItem, JMenuItem resetItem) {
		this.startButton = startButton;
		this.pauseButton = pauseButton;
		this.stopButton = stopButton;
		this.resetButton = resetButton;
		this.startItem = startItem;
		this.pauseItem = pauseItem;
		this.stopItem = stopItem;
		this.resetItem = resetItem;
		this.startButton.setEnabled(true);
		this.startItem.setEnabled(true);
		this.pauseButton.setEnabled(false);
		this.pauseItem.setEnabled(false);
		this.stopButton.setEnabled(false);
		this.stopItem.setEnabled(false);
		this.resetButton.setEnabled(true);
		this.resetItem.setEnabled(true);
		this.simulator = null;
		eventSync = new Object();
		this.eventObjects = new Vector<EventObject>();
		TheaterObserver.setObserver(new RunTheaterObserver());
	}

	public void actionPerformed(ActionEvent e) {
		Play.getPlay().setEventQueueActive(true);
		try {
			JComponent c = (JComponent) e.getSource();
			if ((c == this.startButton || c == this.startItem)
					&& Play.getPlay().getActivePerformance()
							.simulationStopped()) {
				this.handleStart();
			} else if ((c == this.startButton || c == this.startItem)
					&& Play.getPlay().getActivePerformance()
							.simulationSuspended()) {
				this.handleResume();
			} else if (c == this.pauseButton || c == this.pauseItem) {
				this.handleSuspend();
			} else if (c == this.stopButton || c == this.stopItem) {
				this.handleStop();
			} else if (c == this.resetButton || c == this.resetItem) {
				this.handleReset();
			}
		} finally {
			Play.getPlay().setEventQueueActive(false);
		}
	}

	public void handleStart() {
		IPerformance.newLock();
		Play.getPlay().getActivePerformance().startSimulation();
		this.startButton.setEnabled(false);
		this.startItem.setEnabled(false);
		this.pauseButton.setEnabled(true);
		this.pauseItem.setEnabled(true);
		this.stopButton.setEnabled(true);
		this.stopItem.setEnabled(true);
		this.resetButton.setEnabled(true);
		this.resetItem.setEnabled(true);
		this.startButton.setToolTipText(ResourceManager.getResourceManager()
				.getValue("play.toolbar.resume"));
		this.simulator = new RunSimulator();
		this.simulator.startSimulation();
	}

	public void handleSuspend() {
		IPerformance.getPerformance(Play.getPlay().getActivePerformance())
				.suspend();
	}

	public void handleResume() {
		Play.getPlay().getActivePerformance().resumeSimulation();
		this.simulator.resumeSimulation();
		this.startButton.setEnabled(false);
		this.startItem.setEnabled(false);
		this.pauseButton.setEnabled(true);
		this.pauseItem.setEnabled(true);
		this.stopButton.setEnabled(true);
		this.stopItem.setEnabled(true);
		this.resetButton.setEnabled(true);
		this.resetItem.setEnabled(true);
	}

	public void handleAPISuspend() {
		this.startButton.setEnabled(true);
		this.startItem.setEnabled(true);
		this.pauseButton.setEnabled(false);
		this.pauseItem.setEnabled(false);
		this.stopButton.setEnabled(true);
		this.stopItem.setEnabled(true);
		this.resetButton.setEnabled(true);
		this.resetItem.setEnabled(true);
		this.simulator.suspendSimulation();
	}

	public void handleStop() {
		IPerformance.getPerformance(Play.getPlay().getActivePerformance())
				.stop();
	}

	public void handleAPIStop() {
		if (simulator != null) {
			this.simulator.stopSimulation();
			this.simulator = null;
		}
		this.startButton.setEnabled(false);
		this.startItem.setEnabled(false);
		this.pauseButton.setEnabled(false);
		this.pauseItem.setEnabled(false);
		this.stopButton.setEnabled(false);
		this.stopItem.setEnabled(false);
		this.resetButton.setEnabled(true);
		this.resetItem.setEnabled(true);
		this.startButton.setToolTipText(ResourceManager.getResourceManager()
				.getValue("play.toolbar.run"));
	}

	public void handleReset() {
		if (this.simulator != null) {
			this.simulator.stopSimulation();
			this.simulator = null;
		}
		Play.getPlay().getActivePerformance().setStopped();
		this.startButton.setEnabled(true);
		this.startItem.setEnabled(true);
		this.pauseButton.setEnabled(false);
		this.pauseItem.setEnabled(false);
		this.stopButton.setEnabled(false);
		this.stopItem.setEnabled(false);
		this.resetButton.setEnabled(true);
		this.resetItem.setEnabled(true);
		this.startButton.setToolTipText(ResourceManager.getResourceManager()
				.getValue("play.toolbar.run"));
		Play.getPlay().reset();
	}

	public void reset() {
		this.startButton.setEnabled(true);
		this.startItem.setEnabled(true);
		this.pauseButton.setEnabled(false);
		this.pauseItem.setEnabled(false);
		this.stopButton.setEnabled(false);
		this.stopItem.setEnabled(false);
		this.resetButton.setEnabled(true);
		this.resetItem.setEnabled(true);
	}

	public void setDisabled() {
		this.startButton.setEnabled(false);
		this.startItem.setEnabled(false);
		this.pauseButton.setEnabled(false);
		this.pauseItem.setEnabled(false);
		this.stopButton.setEnabled(false);
		this.stopItem.setEnabled(false);
		this.resetButton.setEnabled(true);
		this.resetItem.setEnabled(true);
	}

	public void addActor(Actor actor) {
		this.simulator.addActor(actor);
	}
	
	public void removeActor(Actor actor) {
		this.simulator.removeActor(actor);
	}

	public void handleStageChange(IStage stage) {
		this.simulator.handleStageChange(stage);
	}

	public void newEventObject(EventObject obj) {
		synchronized (eventSync) {
			eventObjects.add(obj);
		}
	}

	public void processEventObjects() {
		Vector<EventObject> copy;
		synchronized (eventSync) {
			copy = new Vector<EventObject>(eventObjects);
			eventObjects.clear();
		}
		for (EventObject obj : copy) {
			obj.call();
		}
	}
}
