package theater_intern;

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.EventQueue;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.concurrent.locks.ReentrantLock;

import model.Play;
import simulation.SimulationManager;
import theater.Actor;
import theater.Performance;
import util.Utils;

/**
 * Interne Umsetzung einer Performance
 * 
 * @author Dietrich Boles, Uni Oldenburg
 * @version 1.0 (12.11.2008)
 * 
 */
public class IPerformance {

	/***************************************************************************
	 * * * * * * * * * * * * * * Umformungsmethoden
	 */
	private static HashMap<Performance, IPerformance> perfMap = new HashMap<Performance, IPerformance>();
	private static HashMap<IPerformance, Performance> iPerfMap = new HashMap<IPerformance, Performance>();

	public static void newPerfIPerf(Performance perf, IPerformance iPerf) {
		IPerformance.perfMap.put(perf, iPerf);
		IPerformance.iPerfMap.put(iPerf, perf);
	}

	public static Performance getPerformance(IPerformance iPerf) {
		return IPerformance.iPerfMap.get(iPerf);
	}

	public static IPerformance getIPerformance(Performance perf) {
		return IPerformance.perfMap.get(perf);
	}

	private static IPerformance defaultPerformance = null;

	/***************************************************************************
	 * * * * * * * * * * * * * * Performancemethoden
	 */

	final public static int STOPPED = 0;
	final public static int RUNNING = 2;
	final public static int SUSPENDED = 3;

	private AudioClip clip;
	private int state;
	private int speed;
	private boolean freeze;

	public static IPerformance getDefaultPerformance() {
		if (IPerformance.defaultPerformance == null) {
			Performance defPerformance = new Performance();
			IPerformance.defaultPerformance = IPerformance
					.getIPerformance(defPerformance);
		}
		return IPerformance.defaultPerformance;
	}

	public IPerformance() {
		this.clip = null;
		this.state = IPerformance.STOPPED;
		this.speed = Performance.DEF_SPEED;
		this.freeze = false;
	}

	public void startSimulation() {
		if (this.state != IPerformance.STOPPED) {
			return;
		}
		IPerformance.getPerformance(this).started();
		this.state = IPerformance.RUNNING;
		TheaterObservable.getObservable().performanceStateChange();
	}

	public void stopSimulation() {
		if (this.state == IPerformance.STOPPED) {
			return;
		}
		SimulationManager.getSimulationManager().handleAPIStop();
		this.state = IPerformance.STOPPED;
		IPerformance.getPerformance(this).stopped();
		TheaterObservable.getObservable().performanceStateChange();
	}

	public void suspendSimulation() {
		if (this.state != IPerformance.RUNNING) {
			return;
		}
		SimulationManager.getSimulationManager().handleAPISuspend();
		this.state = IPerformance.SUSPENDED;
		IPerformance.getPerformance(this).suspended();
		TheaterObservable.getObservable().performanceStateChange();
	}

	public void resumeSimulation() {
		if (this.state != IPerformance.SUSPENDED) {
			return;
		}
		IPerformance.getPerformance(this).resumed();
		this.state = IPerformance.RUNNING;
		TheaterObservable.getObservable().performanceStateChange();
	}

	public void setStopped() {
		if (this.state == IPerformance.STOPPED) {
			return;
		}
		this.state = IPerformance.STOPPED;
		TheaterObservable.getObservable().performanceStateChange();
	}

	public void resetSimulation() {
		if (!this.simulationStopped()) {
			this.stopSimulation();
		}
		Play.getPlay().reset();
	}

	public boolean simulationRunning() {
		return this.state == IPerformance.RUNNING;
	}

	public boolean simulationStopped() {
		return this.state == IPerformance.STOPPED;
	}

	public boolean simulationSuspended() {
		return this.state == IPerformance.SUSPENDED;
	}

	public void freeze() {
		this.freeze = true;
	}

	public void unfreeze() {
		this.freeze = false;
		TheaterObservable.getObservable().importantStateChange();
	}

	public boolean frozen() {
		return this.freeze;
	}

	public void playSound(String soundFile) {
		String dir = Play.getPlay().getDirectory();
		File f = new File(dir + File.separatorChar + "sounds"
				+ File.separatorChar + soundFile);
		if (!f.exists()) {
			throw new IllegalArgumentException();
		}

		try {
			URL url = f.toURI().toURL();
			this.clip = Applet.newAudioClip(url);
			this.clip.play();
		} catch (MalformedURLException e) {
			throw new IllegalArgumentException();
		}
	}

	public int getSimulationSpeed() {
		return this.speed;
	}

	public void setSimulationSpeedAPI(int newSpeed) {
		if (newSpeed != this.speed) {
			this.speed = newSpeed;
			IPerformance.getPerformance(this).speedChanged(this.speed);
			if (Play.getPlay().getPlayFrame() != null) {
				Play.getPlay().getPlayFrame().setSpeed(newSpeed);
			}
		}
	}

	public void setSimulationSpeed(int newSpeed) {
		if (newSpeed != this.speed) {
			this.speed = newSpeed;
		}
	}

	/***************************************************************************
	 * * * * * * * * * * * * * * locking
	 */

	private static ReentrantLock lock = new ReentrantLock();

	public static void newLock() {
		lock = new ReentrantLock();
	}

	public static void lock() {
		if (!EventQueue.isDispatchThread()
				&& Play.getPlay().getActivePerformance().simulationRunning()) {
			lock.lock();
		}
	}

	public static void unlock() {
		if (!EventQueue.isDispatchThread()
				&& Play.getPlay().getActivePerformance().simulationRunning()) {
			lock.unlock();
			Thread.yield();
		}
	}

	public static void forceUnlock() {
		if (!EventQueue.isDispatchThread()) {
			lock.unlock();
			Thread.yield();
		}
	}
}
